use std::{
    collections::HashMap,
    fs::{self, remove_file},
    process::{Command, Stdio},
    time::Duration,
};

use dbus::{
    arg::Variant,
    blocking::{BlockingSender, Connection},
    Message,
};
use log::{debug, error, info, trace};
use resolve_path::PathResolveExt;
use serde::{Deserialize, Serialize};

use crate::tags::Tag;

#[derive(Serialize, Deserialize, Debug)]
pub struct Actions {
    /// command actions
    #[serde(rename = "command")]
    pub commands: Option<Vec<CommandAction>>,
    /// link actions
    #[serde(rename = "link")]
    pub links: Option<Vec<LinkAction>>,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct CommandAction {
    #[serde(default = "whoami::username")]
    user: String,
    command: String,
    pub description: Option<String>,
    pub require: Option<Vec<Tag>>,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct LinkAction {
    pub src: String,
    pub dest: String,
    overwrite: Option<bool>,
}

impl CommandAction {
    pub fn new<S: Into<String>>(command: S, user: S) -> Self {
        Self {
            command: command.into(),
            user: user.into(),
            description: None,
            require: None,
        }
    }

    pub fn run(&self) -> Result<i32, Box<dyn std::error::Error>> {
        trace!("running \"{}\" as {}...", &self.command, &self.user);
        if self.user != whoami::username() {
            Ok(Command::new("sudo")
                .arg("-u")
                .arg(&self.user)
                .arg("--")
                .arg("sh")
                .arg("-c")
                .arg(&self.command)
                .status()?
                .code()
                .unwrap_or(1))
        } else {
            Ok(Command::new("sh")
                .arg("-c")
                .arg(&self.command)
                .status()?
                .code()
                .unwrap_or(1))
        }
    }
}

impl LinkAction {
    pub fn link(&self) -> Result<(), Box<dyn std::error::Error>> {
        trace!(
            "linking from {:?} to {:?}...",
            &self.src.resolve(),
            &self.dest.resolve()
        );
        if let Ok(existing) = fs::read_link(&self.dest.resolve()) {
            if existing == self.src.resolve() {
                debug!("link OK");
                return Ok(());
            } else {
                if self.overwrite.unwrap_or(false) == true {
                    debug!("removing {}...", self.dest);
                    remove_file(self.dest.resolve())?;
                } else {
                    return Err("Destination is linked to a different path".into());
                }
            }
        }
        std::os::unix::fs::symlink(&self.src.resolve(), &self.dest.resolve())?;
        Ok(())
    }
}
