use std::{collections::HashMap, io};

use log::{error, info};
use serde::{Deserialize, Serialize};

use super::action::CommandAction;

#[derive(Serialize, Deserialize, Debug, PartialEq, Eq, Hash)]
pub enum PackageManager {
    #[serde(rename = "pacman")]
    Pacman,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct Package {
    pub name: String,
    pub distro_name_mapping: Option<HashMap<PackageManager, String>>,
}

impl PackageManager {
    pub fn install(&self, packages: Vec<String>) -> Result<bool, Box<dyn std::error::Error>> {
        match &self {
            PackageManager::Pacman => {
                CommandAction::new(
                    format!("pacman -S --noconfirm {}", packages.join(" ")),
                    "root".to_string(),
                )
                .run()?;
            }
        }
        Ok(true)
    }

    pub fn get_available() -> Option<Self> {
        Self::from_str(match whoami::distro().as_str() {
            "Manjaro Linux" => "pacman",
            _ => "unknown"
        })
    }

    pub fn from_str(distro: &str) -> Option<Self> {
        match distro {
            "pacman" => Some(PackageManager::Pacman),
            _ => None,
        }
    }
}

impl Package {
    pub fn get_correct_package_name(&self) -> String {
        if let Some(pm) = PackageManager::get_available() {
            if let Some(mappings) = &self.distro_name_mapping {
                if let Some(name) = mappings.get(&pm) {
                    name.to_string()
                } else {
                    self.name.clone()
                }
            } else {
                self.name.clone()
            }
        } else {
            self.name.clone()
        }
    }

    pub fn install(&self) -> Result<bool, Box<dyn std::error::Error>> {
        if let Some(pm) = PackageManager::get_available() {
            pm.install(vec![self.get_correct_package_name()])?;
        } else {
            error!("no package manager found...");
            return Err(Box::new(io::Error::new(
                io::ErrorKind::NotFound,
                "package manager not found",
            )));
        }
        Ok(false)
    }
}
