use std::{
    env::{current_dir, set_current_dir},
    fs::{create_dir_all, exists, read_to_string},
    path::PathBuf,
};

use log::{debug, info, trace};
use serde::{Deserialize, Serialize};

use crate::{crates::Crate, prelude::abspath};

pub mod git;

#[derive(Serialize, Deserialize, Debug)]
#[serde(default)]
pub struct Source {
    interval: u64,
    git: Option<git::Git>,
    dir: Option<String>,
}

impl Default for Source {
    fn default() -> Self {
        Source {
            interval: 60,
            git: None,
            dir: None,
        }
    }
}

impl Source {
    pub fn available(&self) -> bool {
        if let Some(git) = &self.git {
            trace!("checking git...");
            return git.ensure().is_ok();
        }
        if let Some(dir) = &self.dir {
            return exists(dir).is_ok();
        }
        false
    }

    pub fn go_to_dir(&self) -> Result<(), Box<dyn std::error::Error>> {
        if let Some(git) = &self.git {
            if PathBuf::from(git.repository_path_str()) == current_dir()? {
                return Ok(());
            }
            let dir = git.ensure()?.repository_path()?;
            trace!(
                "setting git dir as cwd... ({}@{}, {})",
                git.url,
                git.branch,
                dir.display()
            );
            set_current_dir(dir)?;
        }
        if let Some(path) = &self.dir {
            set_current_dir(path)?;
        }
        Ok(())
    }

    pub fn get_crates(&self) -> Result<Vec<(PathBuf, Crate)>, Box<dyn std::error::Error>> {
        let mut crates = vec![];
        // get crates (read dir, crates/*/crate.toml)
        for crate_file in glob::glob("crates/*/crate.toml").expect("err") {
            match crate_file {
                Ok(cd) => {
                    debug!("found {}", cd.display());
                    crates.push((cd.clone(), Crate::from_toml_str(&read_to_string(cd)?)?))
                }
                _ => continue,
            }
        }
        Ok(crates)
    }
}
