use serde::{Deserialize, Deserializer, Serialize, Serializer};

#[derive(Debug)]
pub struct Tag {
    pub category: String,
    pub value: String
}

impl<'de> Deserialize<'de> for Tag {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        // First, deserialize the string
        let s: String = Deserialize::deserialize(deserializer)?;

        // Split the string into category and value by ":"
        let parts: Vec<&str> = s.splitn(2, ':').collect();
        if parts.len() != 2 {
            return Err(serde::de::Error::invalid_value(
                serde::de::Unexpected::Str(&s),
                &"a string in the format 'category:value'",
            ));
        }

        // Return a Tag with the split parts
        Ok(Tag {
            category: parts[0].to_string(),
            value: parts[1].to_string(),
        })
    }
}

impl Serialize for Tag {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        serializer.serialize_str(&format!("{}:{}", self.category, self.value))
    }
}

impl Into<String> for Tag {
    fn into(self) -> String {
        self.category + " : " + &self.value
    }
}

impl TryFrom<String> for Tag {
    type Error = std::io::Error;
    fn try_from(value: String) -> Result<Self, Self::Error> {
        let parts: Vec<&str> = value.split(':').collect();
        if parts.len() != 2 {
            Err(std::io::Error::new(std::io::ErrorKind::InvalidData, "could not match string to tag"))
        } else {
            Ok(Self {
                category: parts[0].into(),
                value: parts[1].into()
            })
        }
    }
}

impl Tag {
    pub fn distro() -> Tag {
        Tag {
            category: "distro".into(),
            value: whoami::distro()
        }
    }
}